package org.gcube.portlets.user.dataminermanager.client.rpc;

import java.util.Date;
import java.util.List;
import java.util.Map;

import org.gcube.portlets.user.dataminermanager.client.bean.ComputationStatus;
import org.gcube.portlets.user.dataminermanager.client.bean.CsvMetadata;
import org.gcube.portlets.user.dataminermanager.client.bean.FileMetadata;
import org.gcube.portlets.user.dataminermanager.client.bean.ImportStatus;
import org.gcube.portlets.user.dataminermanager.client.bean.Operator;
import org.gcube.portlets.user.dataminermanager.client.bean.OperatorsClassification;
import org.gcube.portlets.user.dataminermanager.client.bean.ResourceItem;
import org.gcube.portlets.user.dataminermanager.client.bean.output.ImagesResource;
import org.gcube.portlets.user.dataminermanager.client.bean.output.MapResource;
import org.gcube.portlets.user.dataminermanager.client.bean.output.Resource;
import org.gcube.portlets.user.dataminermanager.client.bean.parameters.Parameter;
import org.gcube.portlets.user.dataminermanager.shared.data.ComputationId;
import org.gcube.portlets.user.dataminermanager.shared.data.TableItemSimple;
import org.gcube.portlets.user.dataminermanager.shared.session.UserInfo;
import org.gcube.portlets.user.dataminermanager.shared.workspace.DataMinerWorkArea;
import org.gcube.portlets.user.dataminermanager.shared.workspace.ItemDescription;
import org.gcube.portlets.widgets.wsexplorer.shared.Item;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;

/**
 * 
 * @author Giancarlo Panichi
 * email: <a href="mailto:g.panichi@isti.cnr.it">g.panichi@isti.cnr.it</a> 
 *
 */
public interface DataMinerPortletServiceAsync {
	
	public static DataMinerPortletServiceAsync INSTANCE = (DataMinerPortletServiceAsync) GWT
			.create(DataMinerPortletService.class);

	
	void hello(AsyncCallback<UserInfo> callback);

	void getOperatorsClassifications(
			AsyncCallback<List<OperatorsClassification>> callback);

	void getParameters(Operator operator,
			AsyncCallback<List<Parameter>> callback);

	void startComputation(Operator op, String computationTitle,
			String computationDescription,
			AsyncCallback<ComputationId> asyncCallback);

	void getComputationStatus(ComputationId computationId,
			AsyncCallback<ComputationStatus> asyncCallback);

	void getTableItems(List<String> templates,
			AsyncCallback<List<TableItemSimple>> callback);

	void getFileItems(List<String> templates,
			AsyncCallback<List<TableItemSimple>> callback);

	
	void getResourceByComputationId(ComputationId computationId,
			AsyncCallback<Resource> callback);

	void importTable(CsvMetadata csvMetadata, AsyncCallback<String> callback);

	void getImportsStatus(Date todayDate,
			AsyncCallback<List<ImportStatus>> callback);

	void getImportStatusById(String id, AsyncCallback<ImportStatus> callback);

	void getMapFromMapResource(MapResource mapResource,
			AsyncCallback<Map<String, Resource>> callback);

	void getParametersMapByComputationId(ComputationId computationId,
			AsyncCallback<Map<String, String>> callback);

	void saveImage(ComputationId computationId, ImagesResource imageResource,
			AsyncCallback<String> callback);

	void removeResource(String id, AsyncCallback<Void> callback);

	void removeImport(String id, AsyncCallback<Void> callback);

	void getCsvMetadataFromCsvImporterWizard(AsyncCallback<CsvMetadata> callback);

	void exportResource(String folderId, String fileName,
			ResourceItem resourceItem, AsyncCallback<String> callback);

	void resubmit(ComputationId computationId, AsyncCallback<ComputationId> callback);

	void getFilePathFromImporterWizard(AsyncCallback<FileMetadata> callback);

	void importFile(FileMetadata fileMetadata, AsyncCallback<String> callback);

	void retrieveTableInformation(Item item, AsyncCallback<TableItemSimple> callback);

	void getDataMinerWorkArea(AsyncCallback<DataMinerWorkArea> asyncCallback);

	void getPublicLink(ItemDescription itemDescription,
			AsyncCallback<String> callback);

	void cancelComputation(ComputationId computationId,
			AsyncCallback<String> asyncCallback);

	
}
